package org.yawlfoundation.yawl.elements.data.external;

import java.util.List;

import org.jdom2.Element;
import org.yawlfoundation.yawl.elements.YTask;
import org.yawlfoundation.yawl.elements.data.YParameter;
import org.yawlfoundation.yawl.elements.data.YVariable;
import org.yawlfoundation.yawl.engine.YSpecificationID;
import org.yawlfoundation.yawl.util.JDOMUtil;
import org.yawlfoundation.yawl.util.StringUtil;
import java.util.Iterator;
import message.Message;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.boot.registry.StandardServiceRegistryBuilder;
import org.hibernate.cfg.Configuration;
import org.hibernate.service.ServiceRegistry;

public class MyDBGatewayImpl extends AbstractExternalDBGateway {

    private static SessionFactory factory;
    private static ServiceRegistry registry;
    

    public String getDescription() {
        return "An external database gateway using postgres-jdbc and hibernate";
    }

    @Override
    public Element populateTaskParameter(YTask task, YParameter param,
            Element caseData) {

        Element result = new Element(param.getName());
        try {
            Configuration conf = new Configuration().configure();
              registry = new StandardServiceRegistryBuilder().applySettings(
                    conf.getProperties()).build();
            factory = conf.buildSessionFactory(registry);
        } catch (Throwable ex){
            System.err.println("Failed to create session factory object" + ex);
            throw new ExceptionInInitializerError(ex);
        }
               Session session = factory.openSession();
        Transaction tx = null;
        Short msgId = null;
        try{
            tx=session.beginTransaction();
            List messages = session.createQuery("FROM Message").list();
            for(Iterator iterator = messages.iterator(); iterator.hasNext();){
                Message message = (Message)iterator.next();
                System.out.println("message: "+message.getMessage());
                Element msgElem = (Element) JDOMUtil.stringToElement(StringUtil.wrap(message.getMessage(), "Message")).clone();
                result.addContent(msgElem);
            }
            tx.commit();
        } catch (HibernateException e){
            if(tx != null) tx.rollback();
            e.printStackTrace();
        } finally{
            session.close();
        }
        StandardServiceRegistryBuilder.destroy(registry);
        return result;
    }
    

    @Override
    public void updateFromTaskCompletion(String paramName, Element outputData,
            Element caseData) {
                System.out.println("Message text to be stored: " + outputData.getText());
                String message = outputData.getText();
        try {
            Configuration conf = new Configuration().configure();
              registry = new StandardServiceRegistryBuilder().applySettings(
                    conf.getProperties()).build();
            factory = conf.buildSessionFactory(registry);
        } catch (Throwable ex){
            System.err.println("Failed to create session factory object" + ex);
            throw new ExceptionInInitializerError(ex);
        }
               Session session = factory.openSession();
        Transaction tx = null;
        Short msgId = null;
        try{
            tx=session.beginTransaction();
            Message msg = new Message(message);
            msgId = (Short) session.save(msg);
            tx.commit();
        } catch (HibernateException e){
            if(tx != null) tx.rollback();
            e.printStackTrace();
        } finally{
            session.close();
        }
        StandardServiceRegistryBuilder.destroy(registry);
    }

    @Override
    public Element populateCaseData(YSpecificationID specID, String caseID,
            List<YParameter> inputParams, List<YVariable> localVars,
            Element caseDataTemplate) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public void updateFromCaseData(YSpecificationID specID, String caseID,
            List<YParameter> outputParams, Element updatingData) {
        // TODO Auto-generated method stub

    }

}
